# Copyright 2019-2023 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import urllib.request

from charmhelpers.core import hookenv


def _layer_options():
    # import here, bc not available during unittest
    from charms import layer

    return layer.options().get('ols-http', {})


def port():
    config = hookenv.config()
    port = config['port']
    if not port:
        port = _layer_options().get('port', 8000)
    return port


def metrics_path():
    return _layer_options().get('metrics_path', '/_status/metrics')


def scrape_interval():
    return _layer_options().get('scrape_interval', '1m')


def scrape_timeout():
    return _layer_options().get('scrape_timeout', '10s')


def active_flag_enabled():
    return _layer_options().get('active_flag_enabled', False)


def is_listening(url_path='/', http_port=None, retries=3, wait=5):
    """Check if there is something listening in the specified port/path"""
    # check if the service is running, be pessimistic and retry a couple of
    # times (3 by default)
    if http_port is None:
        http_port = port()
    for _ in range(retries):
        try:
            urllib.request.urlopen('http://localhost:{}{}'.format(http_port, url_path))
            return True
        except Exception:
            time.sleep(wait)
    else:
        return False
