# Copyright 2023 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import yaml
from charmhelpers.core import hookenv, host, templating
from charms.reactive import hook, remove_state, set_state, when, when_not
from ols import base, http


def configure_rsync():
    config = hookenv.config()
    if config["log_hosts_allow"]:
        rsync_config = dict(config)
        rsync_config["base_dir"] = base.base_dir()
        templating.render(
            "lp-archive-rsync.j2",
            "/etc/rsync-juju.d/010-lp-archive.conf",
            rsync_config,
            perms=0o644,
        )
        if not host.service_restart("rsync"):
            raise RuntimeError("Failed to restart rsync")


@when("ols.configured", "memcache.available")
@when_not("service.configured")
def configure(memcache):
    config = dict(hookenv.config())
    config["memcache_servers"] = sorted(
        f"{host}:{port}" for host, port in memcache.memcache_hosts_ports()
    )
    config["layouts"] = yaml.safe_load(config["layouts"])

    hookenv.log("Writing service config.")
    templating.render(
        "config.toml.j2",
        base.service_config_path(),
        config,
        owner="root",
        group=base.user(),
        perms=0o440,
    )

    configure_rsync()

    set_state("service.configured")


@when("service.configured", "ols.wsgi.configured")
def check_is_running():
    if http.is_listening(url_path="/_status/check"):
        hookenv.status_set("active", "Ready")
    else:
        hookenv.status_set("blocked", "Service not running, check logs")


@when("config.changed.build_label")
def build_label_changed():
    remove_state("ols.service.installed")
    remove_state("ols.configured")
    remove_state("service.configured")


@hook("{requires:memcache}-relation-changed")
def memcache_relation_changed(memcache):
    remove_state("service.configured")
