# Copyright 2019-2023 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from charms.reactive import hook, when, when_not, set_state, remove_state

from ols.base import (
    ensure_directories,
    install_payload,
    writable_dirs,
    readonly_dirs,
    user,
    code_dir,
    payload_name,
    version_info_path,
)

from charmhelpers.core import hookenv

# states #


@when_not('ols.service.installed', 'apt.queued_installs')
def install_service():
    writable = writable_dirs()
    readonly = readonly_dirs()
    ensure_directories(readonly, writable, user())
    install_payload(payload_name(), code_dir(), 'root')
    hookenv.status_set('waiting', 'Service installed, but not configured.')
    version_info = version_info_path()
    if os.path.exists(version_info):
        with open(version_info) as version_file:
            hookenv.application_version_set(version_file.read().strip())
    set_state('ols.service.installed')


@hook('upgrade-charm')
def upgrade_charm():
    remove_state('ols.service.installed')  # XXX: remove, only for development.
    remove_state('ols.configured')
    remove_state('service.configured')


@when('config.changed')
def handle_config_changes():
    remove_state('ols.configured')
    remove_state('service.configured')


@when('service.configured')
@when_not('ols.configured')
def reconfigure():
    remove_state('service.configured')


@when('ols.service.installed')
@when_not('apt.queued_installs', 'ols.configured')
def ols_configured():
    set_state('ols.configured')
    hookenv.status_set('waiting', 'ols layer configured')
