# Copyright 2019-2023 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from charms.reactive import helpers, hook, when, set_state, remove_state

from ols import gunicorn, wsgi, base, http

from charmhelpers.core import hookenv

# states #


@hook('upgrade-charm')
def upgrade_charm():
    remove_state('ols.wsgi.configured')
    remove_state('ols.wsgi.restarted')


@when('config.changed')
def handle_config_changes():
    remove_state('ols.wsgi.configured')
    remove_state('ols.wsgi.restarted')


@when('service.configured')
def configure_gunicorn():
    env_extra = base.env_vars()
    gunicorn.configure(
        base.code_dir(), http.port(), base.user(), base.user(), env_extra=env_extra
    )
    set_state('ols.wsgi.configured')


@when('service.configured', 'ols.wsgi.configured')
def needs_restart():

    restart = None
    if helpers.any_file_changed([wsgi.environment_file(), base.version_info_path()]):
        restart = 'hard'
    elif helpers.any_file_changed(wsgi.config_files()):
        restart = 'soft'

    if not restart:
        hookenv.log("Not restarting, no config file changes")
    else:
        hookenv.log("Config files changed, performing {} restart".format(restart))

        # restart gunicorn
        gunicorn.restart(restart == 'soft')
        set_state('ols.wsgi.restarted')
